package bookExamples.ch26Graphics.draw2d.shapes;

import bookExamples.ch26Graphics.draw2d.Containment;
import bookExamples.ch26Graphics.draw2d.ElipseFcn;
import bookExamples.ch26Graphics.draw2d.Fcn1;
import bookExamples.ch26Graphics.draw2d.Vec2d;
import gui.In;
import gui.run.RunButton;
import j2d.ShortImageBean;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;

/**
 * Created by IntelliJ IDEA.
 * User: John Jacocks
 * Date: Dec 1, 2005
 * Time: 5:11:04 PM
 * Note:
 * Modifying class Pentagon by Dave Pavlik
 * to add interfaces for (?Intersects, Movable, &?) Containment
 * for 'prequel' to Final for ECE430, Fall 2005
 * using Triangle2d (by Enrique iturralde, Doug Lyon) as template
 */
public class FillRect2 extends DJShape
        implements PropertyEditor, Containment {
    // possibly? Intersects, Movable,
    private String value = "FillRect2";

    int x1 = 0;
    int y1 = 0;

    int width = 1;
    int height = 1;
    Vec2d center = new Vec2d(0, 0);

    Shape s;

    public FillRect2(int _x1, int _y1, int _x2, int _y2) {
        super(_x1, _y1, _x2, _y2);
        x1 = _x1;
        y1 = _y1;
        int dx = x1 - _x2;
        int dy = y1 - _y2;
        if (_x1 > _x2) x1 = _x2;
        if (_y1 > _y2) y1 = _y2;
        width = dx;
        height = dy;
        center.v[0] = x1 + width / 2;
        center.v[1] = y1 + height / 2;
    }


    public DJShape getInstance(int x1, int y1, int x2, int y2) {
        return new FillRect2(x1, y1, x2, y2);
    }


    public Point getCenter() {
        return new Point((int) center.v[0], (int) center.v[1]);
    }

    public void setCenter() {
        center.v[0] = x1 + width;
        center.v[1] = y1 + height;
    }

    public Component getCustomEditor() {
        JPanel jp = new JPanel();
        jp.setLayout(new FlowLayout());
        jp.add(new RunButton("setForeground") {
            public void run() {
                FillRect2.this.setForeground(In.getColor());

                getPcs().firePropertyChange("size", FillRect2.this, null);
            }
        });
        jp.add(getRunAffinePanel());

        jp.setLayout(new FlowLayout());

        return jp;
    }

    public void setValue(Object value) {
        this.value = (String) value;
    }

    public Object getValue() {
        return value;
    }

    public boolean isPaintable() {
        return false;
    }

    public void paintValue(Graphics gfx, Rectangle box) {
        gfx.setClip(box);
        gfx.drawString("Click to edit...", box.x + 5, box.y + 15);
    }

    public String getJavaInitializationString() {
        return null;
    }

    public String getAsText() {
        return null;
    }

    public void setAsText(String text) throws IllegalArgumentException {
    }

    public String[] getTags() {
        return null;
    }

    public boolean supportsCustomEditor() {
        return true;
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        getPcs().addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        getPcs().removePropertyChangeListener(listener);
    }


    public void draw(Graphics g) {
        Point pc = getCenter();
        Graphics2D g2d = (Graphics2D) g;
        AffineTransform old = g2d.getTransform();
        AffineTransform affineTransform = getAffineTransform();
        g2d.transform(affineTransform);
        Color saveColor = g2d.getColor();
        g2d.setColor(getForeground());

        Shape shape = getShape();
        g2d.draw(shape);
        drawAnnotation(g2d, old, saveColor, shape);
    }

  

    private void drawAnnotation(Graphics2D g2d, AffineTransform old, Color saveColor, Shape s) {
        //   */
        // g2d.fillOval(getX1(), getY1(), 2, 2);
        double x1 = s.getBounds2D().getCenterX();
        double y1 = s.getBounds2D().getCenterY();
        setX1((int) x1);
        setY1((int) y1);
        g2d.drawString("(" + x1 + ", " + x1 + ")", getX1() + 3, getY1() + 3);
        g2d.setTransform(old);

        g2d.setColor(saveColor);
        restoreState(g2d);
    }

    public String toString() {
        Point pc = getCenter();
        return
                "xc,yc,r=" + pc.x + "," + pc.y + "," + width + "," + height;
    }

    public void move(double x, double y) {
        center = new Vec2d(x, y);
    }

    public boolean contains(double x, double y) {
        Shape s = getShape();
        return s.contains(x, y);
    }

    public Shape getShape() {
        Point pc = getCenter();
        x1 = pc.x - width;
        y1 = pc.y - height;

        ElipseFcn cf = new ElipseFcn(pc, width, height);
        int numberOfSides = 4;
        return getShape(numberOfSides, cf);
    }

    private Shape getShape(int numberOfSides, Fcn1 cf) {
        Point2D pa[] = new Point2D[numberOfSides];
        double eps = 1.0 / numberOfSides;
        int i = 0;
        for (double t = .125; i < pa.length; t = t + eps)
            pa[i++] = cf.getPoint(t);
        GeneralPath gp = new GeneralPath();
        gp.moveTo((float) pa[0].getX(), (float) pa[0].getY());
        for (i = 1; i < pa.length; i++) {
            gp.lineTo((float) pa[i].getX(), (float) pa[i].getY());
        }
        gp.closePath();
        AffineTransform at = getAffineTransform();
        return at.createTransformedShape(gp);
    }
}
