package j2d.gui.menu;

import bookExamples.ch26Graphics.draw2d.DrawMain;
import bookExamples.ch26Graphics.draw2d.shapes.ImageShape;
import cr311h3.ImageBrowser;
import gui.ClosableJFrame;
import gui.run.RunMenu;
import gui.run.RunMenuItem;
import j2d.ImageUtils;
import j2d.ShortImageBean;
import j2d.ShortImageBeans;
import j2d.border.BorderPanel;
import j2d.color.rgbImageFilters.LogXformPanel;
import j2d.gui.Main;
import j2d.rotator.Transpose;
import j2d.rotator.Turn180Processor;
import utils.ResourceManager;

import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.util.Observable;
import java.util.Observer;

/**
 * Copyright 2005, DocJava, Inc.
 * User: dlyon
 * Date: Nov 2, 2005
 * Time: 6:13:21 PM
 */
public class ShapeMenu extends RunMenu {
    private Main main;

    public ShapeMenu(Main main) {
        super("Shape");
        this.main = main;
        init();

    }

    private void init() {
        add(new RunMenuItem("append images{alt ctrl A}") {
            public void run() {
                ShapeMenu.appendImages(main);
            }
        });
        add(new RunMenuItem("start draw program") {
            public void run() {
                startDrawMenu();
            }
        });
        add(new RunMenuItem("start MDI draw program") {
            public void run() {
                startMdiDrawMenu();
            }
        });
        addLogPolar();
        add(new RunMenuItem("BorderPanel") {
            public void run() {
                main.addPanel(new BorderPanel(main), getText());
            }
        });
        addTurn();

    }

    private void startMdiDrawMenu() {
        DrawMain dm = new DrawMain();

        Image img = main.getBaseImage();
        if (img == null) return;
        int w = img.getWidth(null);
        int h = img.getHeight(null);
        dm.setLastSelectedShape(new ImageShape(0, 0, w, h, img));
        JPanel jp = new JPanel();
        jp.setLayout(new BorderLayout());
        jp.add(dm, BorderLayout.CENTER);
        jp.setSize(w, h);
        JScrollPane jsp = new JScrollPane();
        jsp.setViewportView(jp);
        main.addPanelNoResizeWindows(jsp, "Draw Panel",dm.getRunMenuBar());

        dm.addCroppedImageObserver(new Observer() {
            public void update(Observable o, Object arg) {
                createNewImageFrame((Image) arg);
            }

        });
        dm.clickMouse();
    }

    private void startDrawMenu() {
        DrawMain dm = new DrawMain();
        ClosableJFrame cf = new ClosableJFrame("DocJavaDraw");
        cf.setJMenuBar(dm.getRunMenuBar());
        Image img = main.getBaseImage();
        if (img == null) return;
        int w = img.getWidth(null);
        int h = img.getHeight(null);
        dm.setLastSelectedShape(new ImageShape(0, 0, w, h, img));
        Container c = cf.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(dm, BorderLayout.CENTER);
        cf.setSize(300, 300);
        cf.setVisible(true);
        dm.addCroppedImageObserver(new Observer() {
            public void update(Observable o, Object arg) {
                createNewImageFrame((Image) arg);
            }

        });
        dm.clickMouse();
    }


    private void createNewImageFrame(Image image) {
        main.addImage(image);
    }

    public void addLogPolar() {
        add(new RunMenuItem("Log Polar transform") {
            public void run() {
                main.addPanel(new LogXformPanel(main), getText());
            }
        });
    }

    public void addTurn() {
        add(new RunMenuItem("Transpose") {
            public void run() {
                main.process(new Transpose());
            }
        });
        add(new RunMenuItem("Turn180") {
            public void run() {
                main.process(new Turn180Processor());
            }
        });
        add(new RunMenuItem("Mirror") {
            public void run() {
                main.process(new j2d.rotator.TurnMirrorProcessor());
            }
        });
        add(new RunMenuItem("UpsideDownMirrorProcessor") {
            public void run() {
                main.process(new j2d.rotator.UpsideDownMirrorProcessor());
            }
        });
        add(new RunMenuItem("ImageBrowser") {
            public void run() {
                ImageBrowser ib = new ImageBrowser();
                ib.start();
            }
        });

    }

    public static void appendImages(Main main) {
        ResourceManager rm = ResourceManager.getResourceManager();
        File f[] = rm.getImages();
        ShortImageBeans sibs = new ShortImageBeans();
        for (int i = 0; i < f.length; i++) {
            final Image image = ImageUtils.getImage(f[i]);
            if (image == null) continue;
            sibs.add(
                    new ShortImageBean(
                            ImageUtils.scaleFast(
                                    image, 64, 64)));
        }
        Image imgs[] = sibs.getImages();
        final int n = (int) Math.sqrt(imgs.length);
        Image img =
                ImageUtils.concatenateImages(
                        imgs, new Dimension(64, 64), n, n);
        main.addImage(img);
    }
}
